#include "test_morpion.hpp"
#include "mcts_two_players.hpp"
#include "morpion.hpp"
#include <iostream>
#include <map>

using namespace std;


namespace game
{
	test_morpion::test_morpion()
	{
		//play();
		playout();
	}
	
	/* Start a complete Player vs Player game */
	void test_morpion::play()
	{
		morpion mor;
		cout << "Player vs Player game!" << endl;
		while(!mor.end_of_game())
		{
			cout << mor << endl;
			cout << "It's " << mor.player_to_string(mor.current_player()) << " turn." << endl;
			map<string, int> movesMap;
			cout << "Number of possible moves: " << mor.number_of_moves() << endl;
			uint64_t possible_moves = mor.get_state().possible_moves;
			for(int i = 0; i < mor.number_of_moves(); i++)
			{
				uint16_t move = possible_moves & ((uint64_t) 15); //15 = 1111 (a move is on 4 bits)
				cout << "Possible move: " << mor.move_to_string(move) << endl;
				movesMap[mor.move_to_string(move)] = i; //In the map: the move as seen by the player and its representation for the computer
				possible_moves = possible_moves >> 4;
			}
			string move;
			cin >> move;
			mor.play(movesMap[move]);
		}
		cout << mor << endl;
		if (mor.won(0))
			cout << mor.player_to_string(0) << " has won" << endl;
		else if (mor.won(1))
			cout << mor.player_to_string(1) << " has won" << endl;
		else
			cout << "Draw! too bad :(" << endl;
		
	}
	
	void test_morpion::playout()
	{
		morpion mor;
		auto mctsPlayer = mcts::make_mcts_two_players(mor, 5000, 0.3, 4); // creation du player a partir du morpion. 5000: ms de reflexion 0.3 fateur d'exploration 4: nb_visit_before_expansion on attend d'etre arrive 4 fois sur un noeud avant de le developper
		cout << "Player vs Computer game!" << endl;
		cout << "Who's first? (Player)" << endl;
		while(!mor.end_of_game())
		{
			cout << mor << endl;
			if(mor.current_player() == 1) //It's circle turn (computer is circle)
			{
				uint16_t move = mctsPlayer.select_move();
				//cout << "Computer played " << mor.move_to_string(move) << endl;
				mor.play(move);
			}
			else //Human turn to play
			{
				cout << "It's your time to shine!" << endl;
				map<string, int> movesMap;
				cout << "Number of possible moves: " << mor.number_of_moves() << endl;
				uint64_t possible_moves = mor.get_state().possible_moves;
				for(int i = 0; i < mor.number_of_moves(); i++)
				{
					uint16_t move = possible_moves & ((uint64_t) 15); //15 = 1111 (a move is on 4 bits)
					cout << "Possible move: " << mor.move_to_string(move) << endl;
					movesMap[mor.move_to_string(move)] = i; //In the map: the move as seen by the player and 
					possible_moves = possible_moves >> 4;
				}
				string move;
				cin >> move;
				mor.play(movesMap[move]);
			}
			
		}
		cout << mor << endl;
		if (mor.won(0))
			cout << mor.player_to_string(0) << " has won" << endl;
		else if (mor.won(1))
			cout << mor.player_to_string(1) << " has won" << endl;
		else
			cout << "Draw! too bad :(" << endl;
	}

}
